# Gallery

Welcome to the TensorFlow.js gallery! This is a collection of TensorFlow.js projects, tutorials, videos, and more.

Please reach out to us if you'd like to put your project on the list.

## Demos & Applications

- [Move Mirror](https://experiments.withgoogle.com/collection/ai/move-mirror/view) - An AI Experiment with Pose Estimation in the Browser using TensorFlow.js
- [Emoji Scavenger Hunt](https://emojiscavengerhunt.withgoogle.com/) - Locate the emoji we show you in the real world with your phone’s camera.
- [Metacar](https://www.metacar-project.com/) - A reinforcement learning environment for self-driving cars in the browser.
- [Evolution Simulator](https://github.com/adityathebe/evolutionSimulator) - Evolution Simulator using NeuroEvolution
- [Play pong with webcam](https://ml4a.github.io/demos/tfjs/regression-pong.html) by Gene Kogan
- [Make music with PoseNet](https://ml4a.github.io/demos/tfjs/posenet-music.html) by Gene Kogan
- [Neural drum machine](https://codepen.io/teropa/pen/JLjXGK) by Tero Parviainen
- [Pose Music](https://codepen.io/teropa/pen/QxLrMp) by Tero Parviainen
- [Neural Arpeggiator](https://codepen.io/teropa/pen/ddqEwj) by Tero Parviainen
- [Neural Melody Autocompletion](https://codepen.io/teropa/pen/gvwwZL) by Tero Parviainen
- [Deep Roll](https://codepen.io/teropa/pen/zpbLOj) by Tero Parviainen
- [Latent Cycles](https://codepen.io/teropa/pen/rdoPbG) by Tero Parviainen
- [PoseNet + Pts.js](https://github.com/williamngan/pts/tree/master/demo/more/tfjs_posenet) - Visualizing pose with pts.js
- [Falling balls + DQN](http://web.sfc.keio.ac.jp/~t15704yn/falling/index.html) - A demo of a DQN agent that learns to dodge falling balls by seann999
- [Tenori Off](https://tenori-off.glitch.me/) - An ML-powered music sequencer by Monica Dinculescu
- [Hello TensorFlow.js](https://hello-tensorflow.glitch.me/) - Polynomial Regression by Monica Dinculescu
- [Simple MNIST GAN](https://mwdchang.github.io/tfjs-gan/) by Daniel Chang
- [Complementary Color Prediction](http://stelling.cc/complementary-color-prediction/) by Roberto Stelling
- [Mars at Home](https://github.com/MarsAtHome/marsjs) - Mars@Home client for Firefox & Chrome - Labels image from Unsplash in browser
- [Nxt Word](https://github.com/rajveermalviya/language-modeling) - Next Word Predictor - by Rajveer Malviya
- [Interactive Classification](https://github.com/poloclub/interactive-classification) - Modify images and see how deep learning classifiers respond
- [Hidden Markov Model with Gaussian emissions](https://github.com/nearform/node-hidden-markov-model-tf) - A trainable Hidden Markov Model with Gaussian emissions using TensorFlow.js. Used in [Node Clinic](https://clinicjs.org/blog/clinic-doctor-just-got-more-advanced-with-tensorflow-js/)
- [Emotion Extractor](https://brendansudol.com/faces/) by Brendan Sudol
- [Aida](https://aida.dor.ai/) Named entity recognition and text classification pipeline for creating chatbots by Rodrigo Pimentel
- [GAN Lab](https://poloclub.github.io/ganlab/) - An Interactive Visualization Tool for Playing with Generative Adversarial Networks (GANs)!
- [Canvas Friends](https://www.y8.com/games/canvas_friends) - Half game, half experiment to see if software can improve the drawing and artistic skills of people.

## Tutorials

These tutorials augment the [official tutorials](https://js.tensorflow.org/tutorials/).

- [A gentle introduction to TensorFlow.js](https://medium.com/tensorflow/a-gentle-introduction-to-tensorflow-js-dba2e5257702) by Zaid Alyafeai
- [Train a model in tf.keras with Colab, and run it in the browser with TensorFlow.js](https://medium.com/tensorflow/train-on-google-colab-and-run-on-the-browser-a-case-study-8a45f9b1474e) by Zaid Alyafeai
- [Python Programming - Deep learning in the browser with TensorFlow.js](https://pythonprogramming.net/deep-learning-browser-introduction-tensorflowjs/) by Harrison Kinsley
- [Basic Tutorial with TensorFlow.js: Linear Regression](https://medium.com/@tristansokol/basic-tutorial-with-tensorflow-js-linear-regression-aa68b16e5b8e) by Tristan Sokol

## Video tutorials
- [The Coding Train - TensorFlow.js Introduction](https://www.youtube.com/playlist?list=PLRqwX-V7Uu6YIeVA3dNxbR9PYj4wV31oQ)
- [The Coding Train - TensorFlow.js Color Classifier](https://www.youtube.com/playlist?list=PLRqwX-V7Uu6bmMRCIoTi72aNWHo7epX4L)
- [Siraj Raval - TensorFlow.js Explained](https://www.youtube.com/watch?v=Nc8kZABv-KE)
- [Siraj Raval - Financial Forecasting using TensorFlow.js](https://www.youtube.com/watch?v=5Uw1iSwvHH8)
- [Siraj Raval - Webcam tracking with TensorFlow.js](https://www.youtube.com/watch?v=9KqNk5keyCc)
- [Angular Firebase - TensorFlow.js Quickstart](https://www.youtube.com/watch?v=Y_XM3Bu-4yc)

## Talks
- [TensorFlow dev summit](https://www.youtube.com/watch?v=YB-kfeNIPCE) - Official TensorFlow.js launch
- [Google I/O, Node.js binding launch](https://www.youtube.com/watch?v=OmofOvMApTU) - Official TensorFlow.js + Node.js launch
- [Deep Learning in JS - Ashi Krishnan - JSConf EU 2018](https://www.youtube.com/watch?v=SV-cgdobtTA)

## Blog posts
- [Move Mirror](https://medium.com/tensorflow/move-mirror-an-ai-experiment-with-pose-estimation-in-the-browser-using-tensorflow-js-2f7b769f9b23?linkId=54484629) - An AI Experiment with Pose Estimation in the Browser using TensorFlow.js
- [Abstract art with ML](https://janhuenermann.com/blog/abstract-art-with-ml) by Jan Huenermann
- [Deep Overwatch with TensorFlow.js](https://medium.com/@farzatv/deepoverwatch-combining-tensorflow-js-overwatch-computer-vision-and-music-1a84d4598bc0) -  combining TensorFlow.js, Overwatch, Computer Vision, and Music by Farza
- [Real-time Human Pose Estimation in the Browser with TensorFlow.js](https://medium.com/tensorflow/real-time-human-pose-estimation-in-the-browser-with-tensorflow-js-7dd0bc881cd5) by Dan Oved
- [Getting Started with TensorFlow.js](https://medium.com/tensorflow/getting-started-with-tensorflow-js-50f6783489b2) by Laurence Moroney
- [Introducing TensorFlow.js](https://medium.com/tensorflow/introducing-tensorflow-js-machine-learning-in-javascript-bf3eab376db) by Josh Gordon and Sara Robinson
- [Solving the Chrome T-Rex Game with Neural Networks, Genetic Algorithms and TensorFlow.js (Chinese: 前端人工智能？TensorFlow.js 学会游戏通关)](https://zhuanlan.zhihu.com/p/35451395) by [@MagicCube](https://github.com/MagicCube)

## Notebooks
- [Introduction to TensorFlow.js](https://beta.observablehq.com/@nsthorat/introduction-to-deeplearn-js) by Nikhil Thorat
- [How to build a Teachable Machine](https://beta.observablehq.com/@nsthorat/how-to-build-a-teachable-machine-with-tensorflow-js) by Nikhil Thorat
- [Smoothgrad in TensorFlow.js](https://beta.observablehq.com/@aman-tiwari/smoothgrad-in-tensorflow-js?linkId=53020429) by Aman Tiwari
- [Visualizing training using TensorFlow.js + MLB data](https://beta.observablehq.com/@nkreeger/visualizing-ml-training-using-tensorflow-js-and-baseball-d) by Nick Kreeger
- [Animation CPPNs](https://beta.observablehq.com/@emilyreif/animation-with-cppns) -  by Emily Reif

## Libraries using TensorFlow.js

- [ML5](https://ml5js.org/) - Friendly machine learning for the web
- [magenta.js](https://magenta.tensorflow.org/js) - Music and Art Generation with Machine Intelligence in the Browser.
- [Handsfree.js](https://github.com/handsfreejs/handsfree) - A library for adding face-controlled pointers to your site.
- [face-api.js](https://github.com/justadudewhohacks/face-api.js) - A JavaScript API for face detection and face recognition in the browser.
- [TensorFlow.js GPU-accelerated t-SNE](https://github.com/tensorflow/tfjs-tsne) - A linear t-SNE for the web.
